﻿namespace anbs_cp.Classes;

/// <summary>
/// Обработчик параметров консоли
/// </summary>
/// <param name="consoleParams">Параметры консоли</param>
/// <param name="caseSensitive">Регистрозависимые ли параметры</param>
public sealed class ConsoleParamsParser(IEnumerable<string> consoleParams, bool caseSensitive = true)
{
    /// <summary>
    /// Массив параметров
    /// </summary>
    private readonly KeyValueList<string, string?> _paramsList = ParseConsoleParams(consoleParams, caseSensitive);

    /// <summary>
    /// Парсер параметров
    /// </summary>
    /// <param name="paramsList">Входящий список параметров</param>
    /// <param name="caseSensitive">Регистрозависимые ли параметры</param>
    /// <returns>Список параметров</returns>
    private static KeyValueList<string, string?> ParseConsoleParams(IEnumerable<string> paramsList,
                                                                    bool caseSensitive = true)
    {
        // Создаю список параметров
        KeyValueList<string, string?> result = [];

        // Заполняю его
        foreach (string consoleParam in paramsList)
        {
            // Индекс знака "="
            int eqPlace = consoleParam.IndexOf('=');

            // Получаю параметр
            string param = eqPlace > -1 ? consoleParam[..eqPlace] : consoleParam;

            // Проверяю, требуются ли регистрозависимые параметры
            if (!caseSensitive)
                // - если нет, то переводим в нижний регистр
                param = param.ToLower();

            // Получаю значение параметра
            string? value = eqPlace == -1 ? null : consoleParam[(eqPlace + 1)..].Trim(['"']);

            // Сохраняю в списке
            result.Add(param.ToLower(), value);
        }

        // Возвращаю значение
        return result;
    }

    /// <summary>
    /// Проверяет наличие параметра
    /// </summary>
    /// <param name="param">Параметр</param>
    /// <returns>Есть ли параметр в списке</returns>
    // ReSharper disable once MemberCanBePrivate.Global
    public bool HasParam(string param)
    {
        // Проверяю регистрозависимость
        if (!caseSensitive)
            // - если нет, то переводим в нижний регистр
            param = param.ToLower();

        // Проверяю наличие параметра
        return _paramsList.Any(keyValue => keyValue.Key == param.ToLower());
    }

    /// <summary>
    /// Проверяет наличие параметров
    /// </summary>
    /// <param name="params">Массив параметров</param>
    /// <returns>Задан ли хотя бы один параметр</returns>
    public bool HasParams(IEnumerable<string> @params) =>
        @params.Aggregate(false, (current, param) => current || HasParam(param));

    /// <summary>
    /// Получает значение параметра,
    /// </summary>
    /// <param name="param">Имя параметра</param>
    /// <param name="default">Значение по умолчанию</param>
    /// <returns>Значение параметра или значение по умолчанию</returns>
    public string GetValue(string param, string @default = "") =>
        !HasParam(param)
            ? @default
            : _paramsList.FirstOrDefault(keyValue => keyValue.Key == param.ToLower()).Value ?? @default;

    /// <summary>
    /// Получает значения параметров из списка.
    /// </summary>
    /// <param name="params">Список параметров</param>
    /// <returns>Список значений параметров</returns>
    public IEnumerable<string> GetValue(IEnumerable<string> @params)
    {
        // Задаю результат
        List<string> result = [];

        // Получаю список параметров
        List<string> paramsList = @params.ToList();

        // Проверяю наличие параметров
        if (!HasParams(paramsList))
            // - если нет, возвращаю значение по умолчанию
            return [];

        // Для каждого параметра, если задано значение - добавляю его в результат
        result.AddRange(paramsList.Where(HasParam).Select(param => GetValue(param))
                                  .Where(value => !string.IsNullOrWhiteSpace(value)));

        // Возвращаю результат
        return result;
    }

    /// <summary>
    /// Получает значение первого заданного параметра и возвращает значение по умолчанию, если ни один из параметров не задан.
    /// </summary>
    /// <param name="params">Список параметров</param>
    /// <param name="default">Значение по умолчанию</param>
    /// <returns>Значение первого заданного параметра из списка или значение по умолчанию</returns>
    public string GetValue(IEnumerable<string> @params, string @default) =>
        GetValue(@params).FirstOrDefault() ?? @default;

    /// <summary>
    /// Получает список всех параметров
    /// </summary>
    /// <returns>Список всех параметров</returns>
    public List<string> GetParamsList() =>
        _paramsList.Select(static keyValue => keyValue.Key.ToLower()).ToList();
}