﻿using System.Collections;

using anbs_cp.Interfaces;
using anbs_cp.Structs;

namespace anbs_cp.Classes;

/// <summary>
/// Упорядоченный список из пары ключ-значение
/// </summary>
/// <typeparam name="TV">Тип значения</typeparam>
// ReSharper disable once ClassCanBeSealed.Global
public class KeyValueOrderedList<TV>: IEnumerable<KeyValueOrdered<TV>>, ISerializable
{
    /// <summary>
    /// Хранение значений
    /// </summary>
    private readonly SortedSet<KeyValueOrdered<TV>> _list = [];

    #region Свойства
    /// <summary>
    /// Список ключей из списка
    /// </summary>
    public IEnumerable<string> Keys => GetKeys();
    #endregion

    #region Методы
    /// <summary>
    /// Получает список ключей
    /// </summary>
    /// <returns>Список ключей</returns>
    private IEnumerable<string> GetKeys () => from keyValue in _list where keyValue.Key is not null select keyValue.Key;

    /// <summary>
    /// Добавляет в список параметр
    /// </summary>
    /// <param name="keyValue">Параметр</param>
    public void Add (KeyValueOrdered<TV> keyValue)
    {
        // Проверяет, существует ли ключ в списке
        if (Contains(keyValue.Key))
            ChangeValue(keyValue);

        // Добавляем в список
        _list.Add(keyValue);
    }

    /// <summary>
    /// Добавляет в список параметр
    /// </summary>
    /// <param name="key">Ключ параметра</param>
    /// <param name="value">Значение</param>
    public void Add (string key, TV value) => Add(new(key, value));

    /// <summary>
    /// Добавляет в список некоторый набор элементов
    /// </summary>
    /// <param name="list">Некоторый набор элементов</param>
    public void AddRange (IEnumerable<KeyValueOrdered<TV>> list)
    {
        // Получаю список
        List<KeyValueOrdered<TV>> keyValues = list.ToList();

        // Для каждого элемента списка
        foreach (KeyValueOrdered<TV> keyValue in keyValues)
            // - добавляю его в список
            Add(keyValue);
    }

    /// <summary>
    /// Изменяет значение
    /// </summary>
    /// <param name="keyValue">Новое значение</param>
    // ReSharper disable once MemberCanBePrivate.Global
    public void ChangeValue (KeyValueOrdered<TV> keyValue)
    {
        // Если такой ключ не существует
        if (!Contains(keyValue.Key))
        {
            // - тогда добавляю новое значение
            Add(keyValue);

            // - прерываю
            return;
        }

        // Удаляем существующее
        _list.RemoveWhere(item => item.Key == keyValue.Key);

        // Добавляем новое
        _list.Add(keyValue);
    }

    /// <summary>
    /// Изменяет значение
    /// </summary>
    /// <param name="key">Ключ</param>
    /// <param name="newValue">Новое значение</param>
    public void ChangeValue (string key, TV newValue) => ChangeValue(new(key, newValue));

    /// <summary>
    /// Получает элемент по ключу
    /// </summary>
    /// <param name="key">Ключ</param>
    /// <returns>Элемент <see cref="KeyValue{TK,TV}"/></returns>
    // ReSharper disable once MemberCanBePrivate.Global
    public KeyValueOrdered<TV>? GetItem (string key)
    {
        // Ищем элемент в списке
        foreach (KeyValueOrdered<TV> keyValueItem in _list.Where(item => item.Key == key))
            // - возвращаем его при нахождении
            return keyValueItem;

        // Элемент не найден -- вывожу null
        return null;
    }

    /// <summary>
    /// Получает значение
    /// </summary>
    /// <param name="key">Ключ</param>
    /// <returns>Значение</returns>
    public TV? GetValue (string key)
    {
        // Если такой ключ не существует
        if (!Contains(key))
            // Тогда возвращаю значение по умолчанию
            return default;

        // Получаю элемент
        KeyValueOrdered<TV> keyValue = GetItem(key) ?? new();

        // Вывожу значение
        return keyValue.Value;
    }

    /// <summary>
    /// Удаляет элемент из списка
    /// </summary>
    /// <param name="keyValue">Элемент</param>
    // ReSharper disable once MemberCanBePrivate.Global
    public void Remove (KeyValueOrdered<TV> keyValue) => _list.Remove(keyValue);

    /// <summary>
    /// Удаляет элемент из списка
    /// </summary>
    /// <param name="key">Ключ элемента</param>
    public void Remove (string key) => Remove(GetItem(key) ?? new());

    /// <summary>
    /// Проверяет, содержится ли элемент в списке
    /// </summary>
    /// <param name="keyValue">Элемент</param>
    /// <returns>Результат проверки</returns>
    // ReSharper disable once MemberCanBePrivate.Global
    public bool Contains (KeyValueOrdered<TV> keyValue) => _list.Contains(keyValue);

    /// <summary>
    /// Проверяет, содержится ли элемент в списке
    /// </summary>
    /// <param name="key">Ключ элемента</param>
    /// <returns>Результат проверки</returns>
    // ReSharper disable once NullableWarningSuppressionIsUsed
    // ReSharper disable once MemberCanBePrivate.Global
    public bool Contains (string key) => Keys.Any(keyName => keyName == key);

    /// <summary>
    /// Очистка списка
    /// </summary>
    public void Clear ()
    {
        // Очищаю список
        _list.Clear();
    }
    #endregion

    #region Реализация интерфейса IEnumerable<KeyValue<TK, TV>>
    /// <inheritdoc />
    public IEnumerator<KeyValueOrdered<TV>> GetEnumerator () => _list.GetEnumerator();

    /// <inheritdoc />
    IEnumerator IEnumerable.GetEnumerator () => GetEnumerator();
    #endregion

    #region Реализация интерфейса ISerializable
    /// <inheritdoc />
    public string Serialize ()
    {
        // Создаю результат
        List<string> result = [];

        // Добавляю сериализованные значения
        result.AddRange(_list.Select(static item => item.Serialize()));

        // Вывожу результат
        return new NewtonsoftJsonSerializer().Serialize(result);
    }

    /// <inheritdoc />
    public void Deserialize (string json)
    {
        // Создаю результат
        List<string> result = new NewtonsoftJsonSerializer().Deserialize<List<string>>(json) ?? [];

        // Очищаю список
        _list.Clear();

        // Пробегаю все значения
        foreach (string itemString in result)
        {
            // - создаю новое значение
            KeyValueOrdered<TV> item = new();

            // - десериализую в него элемента
            item.Deserialize(itemString);

            // - добавляю получившееся в список
            _list.Add(item);
        }
    }
    #endregion
}