﻿using System.Diagnostics.CodeAnalysis;

using anbs_cp.Classes;
using anbs_cp.Enums;

namespace anbs_cp.Extensions;

/// <summary>
/// Расширение строк
/// </summary>
public static class StringExtensions
{
    /// <summary>
    /// Проверяет строку на пустоту
    /// </summary>
    /// <param name="s">Строка</param>
    /// <returns>Строка пуста (null) или содержит только пробелы</returns>
    public static bool IsNullOrWhiteSpace ([NotNullWhen(false)] this string? s) => s == null || s.Trim().Length == 0;

    /// <summary>
    /// Преобразует текстовое имя параметра перечисления <see cref="T"/> в элемент перечисления.
    /// Если такой элемент не найден или при ошибке перевода вернётся элемент по умолчанию <paramref name="defaultValue"/>.
    /// </summary>
    /// <param name="s">Строка</param>
    /// <param name="defaultValue">Значение по умолчанию</param>
    /// <typeparam name="T">Тип перечисления</typeparam>
    /// <returns>Элемент перечисления <see cref="T"/></returns>
    public static T ParseToEnum<T> (this string s, T defaultValue)
    {
        try
        {
            return (T)Enum.Parse(typeof(T), s, true);
        }
        catch (ArgumentException)
        {
            return defaultValue;
        }
    }

    /// <summary>
    /// Приводит строку <paramref name="s"/> к виду "Первая буква заглавная, а все остальные -- строчные"
    /// </summary>
    /// <param name="s">Строка для перевода</param>
    /// <returns>Приведённая строка</returns>
    public static string ToFirstUpperCase (this string s)
    {
        // Если строка пуста
        if (s.IsNullOrWhiteSpace())
            // - то возвращаю её
            return s;

        // Задаю результат и устанавливаю первую букву заглавной
        string result = $"{s[0]}".ToUpper();

        // Если длина s больше, чем 1 символ
        if (s.Length > 1)
            // - то добавляю в результат оставшиеся символы, приведя их к LowerCase
            result += s[1..].ToLower();

        // Вывожу результат
        return result;
    }

    /// <summary>
    /// Трансформация текста <paramref name="s"/> по правилам <paramref name="type"/>
    /// </summary>
    /// <param name="s">Текст</param>
    /// <param name="type">Правила преобразования</param>
    /// <returns>Преобразованный текст</returns>
    public static string Transform (this string s, ETextTransform type) =>
        type switch
        {
            ETextTransform.None => s,
            ETextTransform.LowerCase => s.ToLower(),
            ETextTransform.UpperCase => s.ToUpper(),
            ETextTransform.FirstUpper => s.ToFirstUpperCase(),
            var _ => s
        };

    /// <summary>
    /// Форматирует строку <paramref name="s"/>, заменяя все совпадения
    /// по ключам из <paramref name="replaceList"/> соответствующими значениями из <paramref name="replaceList"/>.
    /// Т.е. например, исходная строка «Мой дядя #Name был самых честных правил», массив замен содержал строку
    /// «#Name» => «Юра», в итоге будет «Мой дядя Юра был самых честных правил».
    /// </summary>
    /// <param name="s">Строка для замены</param>
    /// <param name="replaceList">Список замен</param>
    /// <param name="comparisonType">Региональные параметры, регистр и правила сортировки</param>
    /// <returns>Заменённая строка</returns>
    public static string Format (this string s, KeyValueOrderedList<string> replaceList,
        StringComparison comparisonType = StringComparison.CurrentCulture) => replaceList.Aggregate(s,
        (current, replaceItem) => current.Replace(replaceItem.Key, replaceItem.Value, comparisonType));
}