﻿using anbs_cp.Classes;
using anbs_cp.Enums;
using anbs_cp.Exceptions;
using anbs_cp.Interfaces;

namespace anbs_cp.Structs;

/// <summary>
/// Упорядоченная пара ключ-значение
/// </summary>
/// <typeparam name="TValue">Тип значения</typeparam>
/// <param name="key">Ключ</param>
/// <param name="value">Значение</param>
public struct KeyValueOrdered<TValue> (string key, TValue? value): ISerializable, IComparable<KeyValueOrdered<TValue>>
{
    #region Свойства
    /// <summary>
    /// Ключ
    /// </summary>
    public string Key { get; private set; } = key;

    /// <summary>
    /// Значение
    /// </summary>
    // ReSharper disable once MemberCanBePrivate.Global
    public TValue? Value { get; set; } = value;
    #endregion

    #region Методы
    /// <summary>
    /// Получает ключ-значение по умолчанию
    /// </summary>
    /// <returns>Ключ-значение по умолчанию</returns>
    public static KeyValueOrdered<TValue> GetDefault () => new();
    #endregion

    #region Конвертация в KeyValue<TKey, TValue>
    /// <summary>
    /// Переводит тип <see cref="KeyValueOrdered{TValue}"/> в тип <see cref="KeyValue{TK,TV}"/>
    /// </summary>
    /// <param name="keyValue">Тип <see cref="KeyValueOrdered{TValue}"/></param>
    /// <returns>Тип <see cref="KeyValue{TK,TV}"/></returns>
    public static KeyValue<string, TValue> ToKeyValue (KeyValueOrdered<TValue> keyValue) =>
        new(keyValue.Key, keyValue.Value);

    /// <summary>
    /// Переводит тип <see cref="KeyValue{TK,TV}"/> в тип <see cref="KeyValueOrdered{TValue}"/>
    /// </summary>
    /// <param name="keyValue">Тип <see cref="KeyValue{TK,TV}"/></param>
    /// <returns>Тип <see cref="KeyValueOrdered{TValue}"/></returns>
    public static KeyValueOrdered<TValue> ToOrderedKeyValue (KeyValue<string, TValue> keyValue) =>
        new(keyValue.Key, keyValue.Value);
    #endregion

    #region Реализация интерфейса ISerializable
    /// <inheritdoc />
    public readonly string Serialize ()
    {
        // Получаю serialized-значение ключа
        string keySerialized = Key;

        // Исключаю из этого значения = (заменяя на %EQ%)
        keySerialized = keySerialized.Replace("=", "%EQ%");

        // Получаю serialized-значение значения
        string valueSerialized = new NewtonsoftJsonSerializer().Serialize(Value);

        // Исключаю из этого значения = (заменяя на %EQ%)
        valueSerialized = valueSerialized.Replace("=", "%EQ%");

        // Вывожу результат
        return $"{keySerialized}={valueSerialized}";
    }

    /// <inheritdoc />
    public void Deserialize (string json)
    {
        // Разделяю сериализованную строку вида КЛЮЧ=ЗНАЧЕНИЕ на ключ и значение
        string[] splited = json.Split('=');

        // Проверяю количество частей (должно быть 2)
        if (splited.Length != 2)
            throw new SerializeException(ESerializeExceptionAction.Deserialize, json,
                "Неверное количество частей! Наверное, лишнее =?");

        // Получаю сериализованное представление ключа
        string keySerialized = splited[0].Replace("%EQ%", "=");

        // Получаю сериализованное представление значения
        string valueSerialized = splited[1].Replace("%EQ%", "=");

        // Десериализую ключ
        Key = keySerialized;

        // Десериализую значение
        Value = new NewtonsoftJsonSerializer().Deserialize<TValue>(valueSerialized);
    }
    #endregion

    #region Реализация интерфейса IComparable<KeyValueOrdered<TValue>>
    /// <inheritdoc />
    public int CompareTo (KeyValueOrdered<TValue> other) =>
        // ReSharper disable once SuspiciousTypeConversion.Global
        string.Compare(Key, other.Key, StringComparison.CurrentCultureIgnoreCase);
    #endregion
}